/*
 * Name:
 *     Synchronizing tasks with a semaphore (signal an event)
 *
 * Description:
 *     - a FreeRTOS example application for the Silicon Labs
 *       EFM32 Giant Gecko Starter Kit (STK3700A)
 *     - There are two tasks. One is signaling an event by using a semaphore and
 *       the other is waiting for this event.
 *     - This example can be solved in two ways:
 *       - If USING_BUTTONS is defined as 0 (this is the default) then the
 *         signaling task is supposed to signal the event once in every second
 *       - If USING_BUTTONS is defined as 1 then the signaling task is supposed
 *         to signal the event only when a button (PB0) is pressed
 *
 * Created by:
 *     NASZALY Gabor <naszaly@mit.bme.hu>
 *
 * Last modified on:
 *     2019-02-19
*/

#define USING_BUTTONS 0

/* --------------------------------------------------------------------------
   INCLUDES
-------------------------------------------------------------------------- */

// Application specific includes
#include <stdio.h>
#if (RETARGET_IMPLEMENTATION == 1)
    #include <retargetserial.h>
#elif (RETARGET_IMPLEMENTATION == 2)
    #include <retargetswo.h>
#endif
#if USING_BUTTONS
    // TODO: include bsp_stk_buttons.h for handling buttons on the board
#endif

// FreeRTOS includes
#include "FreeRTOS.h"
#include "task.h"
// TODO: include header for semaphore API

/* --------------------------------------------------------------------------
   DEFINES
-------------------------------------------------------------------------- */

#define  mainTASK_SIGNAL_PRODUCER_PRIORITY    ( tskIDLE_PRIORITY + 2 )
#define  mainTASK_SIGNAL_PRODUCER_STACK_SIZE  configMINIMAL_STACK_SIZE

#define  mainTASK_SIGNAL_CONSUMER_PRIORITY    ( tskIDLE_PRIORITY + 1 )
#define  mainTASK_SIGNAL_CONSUMER_STACK_SIZE  configMINIMAL_STACK_SIZE

/* --------------------------------------------------------------------------
   VARIABLES
-------------------------------------------------------------------------- */

// The binary semaphore used to signal the event
SemaphoreHandle_t event;

/* --------------------------------------------------------------------------
   FUNCTION DECLARATIONS
-------------------------------------------------------------------------- */

static void prvTaskSignalProducer(void *pvParam);
static void prvTaskSignalConsumer(void *pvParam);

/* --------------------------------------------------------------------------
   MAIN
-------------------------------------------------------------------------- */

int main(void) {

    // Initialize starter kit
    #if (RETARGET_IMPLEMENTATION == 1)
        RETARGET_SerialInit();
        RETARGET_SerialCrLf(1);
    #elif (RETARGET_IMPLEMENTATION == 2)
        RETARGET_SwoInit();
    #endif

    #if USING_BUTTONS
        /*
         * TODO: initialize buttons
         *
         * Use BSP_ButtonsInit().
        */
    #endif

    // Initialize FreeRTOS

        // TODO: create the binary semaphore for signaling the event.

        // Create the signal producer task
        xTaskCreate(
                prvTaskSignalProducer,
                "Signal Producer",
                mainTASK_SIGNAL_PRODUCER_STACK_SIZE,
                NULL,
                mainTASK_SIGNAL_PRODUCER_PRIORITY,
                NULL);

        // Create the signal consumer task
        xTaskCreate(
                prvTaskSignalConsumer,
                "Signal Consumer",
                mainTASK_SIGNAL_CONSUMER_STACK_SIZE,
                NULL,
                mainTASK_SIGNAL_CONSUMER_PRIORITY,
                NULL);

        // Start the scheduler
        vTaskStartScheduler();

    // Prevent compiler warning
    return 0;
}

/* --------------------------------------------------------------------------
   FUNCTION DEFINITIONS
-------------------------------------------------------------------------- */

/*
 * This task produces the events
 *
 * TODO:
 *
 * TODO: change the code from producing the event unconditionally in every
 * second to produce the event only when a button has been just pressed. Use
 * function BSP_ButtonGet(). (The buttons are low active.)
*/
static void prvTaskSignalProducer(void *pvParam) {

    while (1) {

        #if USING_BUTTONS
            // TODO: release the semaphore every time PB0 is pressed

        #else
            // TODO: release the semaphore at every second


        #endif

    }
}

/*
 * This task consumes the events
*/
static void prvTaskSignalConsumer(void *pvParam) {
    while (1) {
        // TODO: try to take the semaphore and print out a message
        // if the semaphore was received


    }
}
