/*
 * Name:
 *     Two independent threads
 *
 * Description:
 *     - a FreeRTOS example application for the Silicon Labs
 *       EFM32 Giant Gecko Starter Kit (STK3700A)
 *     - there are two independent tasks periodically printing out something
 *     - the aim is to test if FreeRTOS is able to schedule more than one task
 *       "simultaneously" (multi-tasking proof of concept)
 *
 * Created by:
 *     NASZALY Gabor <naszaly@mit.bme.hu>
 *
 * Last modified on:
 *     2019-02-19
*/

/* --------------------------------------------------------------------------
   INCLUDES
-------------------------------------------------------------------------- */

// Application specific includes
#include <stdio.h>
#if (RETARGET_IMPLEMENTATION == 1)
    #include <retargetserial.h>
#elif (RETARGET_IMPLEMENTATION == 2)
    #include <retargetswo.h>
#endif

// FreeRTOS includes
#include "FreeRTOS.h"
#include "task.h"

/* --------------------------------------------------------------------------
   DEFINES
-------------------------------------------------------------------------- */

#define  mainTASK_HI_PRIORITY    ( tskIDLE_PRIORITY + 2 )
#define  mainTASK_HI_STACK_SIZE  configMINIMAL_STACK_SIZE

#define  mainTASK_LO_PRIORITY    ( tskIDLE_PRIORITY + 1 )
#define  mainTASK_LO_STACK_SIZE  configMINIMAL_STACK_SIZE

/* --------------------------------------------------------------------------
   VARIABLES
-------------------------------------------------------------------------- */

/* --------------------------------------------------------------------------
   FUNCTION DECLARATIONS
-------------------------------------------------------------------------- */

static void prvTaskHi(void *pvParam);
static void prvTaskLo(void *pvParam);

/* --------------------------------------------------------------------------
   MAIN
-------------------------------------------------------------------------- */

int main(void) {

    // Initialize starter kit
    #if (RETARGET_IMPLEMENTATION == 1)
        RETARGET_SerialInit();
        RETARGET_SerialCrLf(1);
    #elif (RETARGET_IMPLEMENTATION == 2)
        RETARGET_SwoInit();
    #endif

    // Initialize FreeRTOS

        // TODO: create the high priority task --> xTaskCreate()

        // TODO: create the low priority task --> xTaskCreate()

        vTaskStartScheduler();

    // Prevent compiler warning
    return 0;
}

/* --------------------------------------------------------------------------
   FUNCTION DEFINITIONS
-------------------------------------------------------------------------- */

/*
 * TODO: implement the high priority task as an endless loop
 *     - Print out a message which:
 *          1: says that it belongs to the high priority task
 *          2: contains a counter which is incremented within the loop
 *     - Wait 1 second:
 *       - call vTaskDelay()
 *       - configTICK_RATE_HZ can be used in the parameter
*/
static void prvTaskHi(void *pvParam) {
}

/*
 * TODO: implement the low priority task as an endless loop
 *     - Print out a message which:
 *          1: says that it belongs to the low priority task
 *          2: contains a counter which is incremented within the loop
 *     - Wait 0.5 second:
 *       - call vTaskDelay()
 *       - configTICK_RATE_HZ can be used in the parameter
*/
static void prvTaskLo(void *pvParam) {
}
